# pylint: disable=missing-class-docstring, invalid-name
"""
ESI models for providers

These models have been generated from stubs of compatibility date 2026-03-20.
Make sure to update them as compatibility date changed
"""

from datetime import datetime
from typing import Literal, TypedDict


class CharactersCharacterIdAssetsGetItem(TypedDict):
    is_blueprint_copy: bool | None
    is_singleton: bool
    item_id: int
    location_flag: Literal[
        "AssetSafety",
        "AutoFit",
        "BoosterBay",
        "CapsuleerDeliveries",
        "Cargo",
        "CorporationGoalDeliveries",
        "CorpseBay",
        "Deliveries",
        "DroneBay",
        "ExpeditionHold",
        "FighterBay",
        "FighterTube0",
        "FighterTube1",
        "FighterTube2",
        "FighterTube3",
        "FighterTube4",
        "FleetHangar",
        "FrigateEscapeBay",
        "Hangar",
        "HangarAll",
        "HiSlot0",
        "HiSlot1",
        "HiSlot2",
        "HiSlot3",
        "HiSlot4",
        "HiSlot5",
        "HiSlot6",
        "HiSlot7",
        "HiddenModifiers",
        "Implant",
        "InfrastructureHangar",
        "LoSlot0",
        "LoSlot1",
        "LoSlot2",
        "LoSlot3",
        "LoSlot4",
        "LoSlot5",
        "LoSlot6",
        "LoSlot7",
        "Locked",
        "MedSlot0",
        "MedSlot1",
        "MedSlot2",
        "MedSlot3",
        "MedSlot4",
        "MedSlot5",
        "MedSlot6",
        "MedSlot7",
        "MobileDepotHold",
        "MoonMaterialBay",
        "QuafeBay",
        "RigSlot0",
        "RigSlot1",
        "RigSlot2",
        "RigSlot3",
        "RigSlot4",
        "RigSlot5",
        "RigSlot6",
        "RigSlot7",
        "ShipHangar",
        "Skill",
        "SpecializedAmmoHold",
        "SpecializedAsteroidHold",
        "SpecializedCommandCenterHold",
        "SpecializedFuelBay",
        "SpecializedGasHold",
        "SpecializedIceHold",
        "SpecializedIndustrialShipHold",
        "SpecializedLargeShipHold",
        "SpecializedMaterialBay",
        "SpecializedMediumShipHold",
        "SpecializedMineralHold",
        "SpecializedOreHold",
        "SpecializedPlanetaryCommoditiesHold",
        "SpecializedSalvageHold",
        "SpecializedShipHold",
        "SpecializedSmallShipHold",
        "StructureDeedBay",
        "SubSystemBay",
        "SubSystemSlot0",
        "SubSystemSlot1",
        "SubSystemSlot2",
        "SubSystemSlot3",
        "SubSystemSlot4",
        "SubSystemSlot5",
        "SubSystemSlot6",
        "SubSystemSlot7",
        "Unlocked",
        "Wardrobe",
    ]
    location_id: int
    location_type: Literal["station", "solar_system", "item", "other"]
    quantity: int
    type_id: int


class CharactersCharacterIdAssetsLocationsPostItem_Position(TypedDict):
    """Manually generated model"""

    x: int
    y: int
    z: int


class CharactersCharacterIdAssetsLocationsPostItem(TypedDict):
    """Manually generated model"""

    item_id: int
    position: CharactersCharacterIdAssetsLocationsPostItem_Position


class CharactersCharacterIdNotificationsGetItem(TypedDict):
    is_read: bool | None
    notification_id: int
    sender_id: int
    sender_type: Literal["character", "corporation", "alliance", "faction", "other"]
    text: str | None
    timestamp: datetime
    type: Literal[
        "AcceptedAlly",
        "AcceptedSurrender",
        "AgentRetiredTrigravian",
        "AllAnchoringMsg",
        "AllMaintenanceBillMsg",
        "AllStrucInvulnerableMsg",
        "AllStructVulnerableMsg",
        "AllWarCorpJoinedAllianceMsg",
        "AllWarDeclaredMsg",
        "AllWarInvalidatedMsg",
        "AllWarRetractedMsg",
        "AllWarSurrenderMsg",
        "AllianceCapitalChanged",
        "AllianceWarDeclaredV2",
        "AllyContractCancelled",
        "AllyJoinedWarAggressorMsg",
        "AllyJoinedWarAllyMsg",
        "AllyJoinedWarDefenderMsg",
        "BattlePunishFriendlyFire",
        "BillOutOfMoneyMsg",
        "BillPaidCorpAllMsg",
        "BountyClaimMsg",
        "BountyESSShared",
        "BountyESSTaken",
        "BountyPlacedAlliance",
        "BountyPlacedChar",
        "BountyPlacedCorp",
        "BountyYourBountyClaimed",
        "BuddyConnectContactAdd",
        "CharAppAcceptMsg",
        "CharAppRejectMsg",
        "CharAppWithdrawMsg",
        "CharLeftCorpMsg",
        "CharMedalMsg",
        "CharTerminationMsg",
        "CloneActivationMsg",
        "CloneActivationMsg2",
        "CloneMovedMsg",
        "CloneRevokedMsg1",
        "CloneRevokedMsg2",
        "CombatOperationFinished",
        "ContactAdd",
        "ContactEdit",
        "ContainerPasswordMsg",
        "ContractRegionChangedToPochven",
        "CorpAllBillMsg",
        "CorpAppAcceptMsg",
        "CorpAppInvitedMsg",
        "CorpAppNewMsg",
        "CorpAppRejectCustomMsg",
        "CorpAppRejectMsg",
        "CorpBecameWarEligible",
        "CorpDividendMsg",
        "CorpFriendlyFireDisableTimerCompleted",
        "CorpFriendlyFireDisableTimerStarted",
        "CorpFriendlyFireEnableTimerCompleted",
        "CorpFriendlyFireEnableTimerStarted",
        "CorpKicked",
        "CorpLiquidationMsg",
        "CorpNewCEOMsg",
        "CorpNewsMsg",
        "CorpNoLongerWarEligible",
        "CorpOfficeExpirationMsg",
        "CorpStructLostMsg",
        "CorpTaxChangeMsg",
        "CorpVoteCEORevokedMsg",
        "CorpVoteMsg",
        "CorpWarDeclaredMsg",
        "CorpWarDeclaredV2",
        "CorpWarFightingLegalMsg",
        "CorpWarInvalidatedMsg",
        "CorpWarRetractedMsg",
        "CorpWarSurrenderMsg",
        "CorporationGoalClosed",
        "CorporationGoalCompleted",
        "CorporationGoalCreated",
        "CorporationGoalExpired",
        "CorporationGoalLimitReached",
        "CorporationGoalNameChange",
        "CorporationLeft",
        "CustomsMsg",
        "DailyItemRewardAutoClaimed",
        "DeclareWar",
        "DistrictAttacked",
        "DustAppAcceptedMsg",
        "ESSMainBankLink",
        "EntosisCaptureStarted",
        "ExpertSystemExpired",
        "ExpertSystemExpiryImminent",
        "FWAllianceKickMsg",
        "FWAllianceWarningMsg",
        "FWCharKickMsg",
        "FWCharRankGainMsg",
        "FWCharRankLossMsg",
        "FWCharWarningMsg",
        "FWCorpJoinMsg",
        "FWCorpKickMsg",
        "FWCorpLeaveMsg",
        "FWCorpWarningMsg",
        "FacWarCorpJoinRequestMsg",
        "FacWarCorpJoinWithdrawMsg",
        "FacWarCorpLeaveRequestMsg",
        "FacWarCorpLeaveWithdrawMsg",
        "FacWarDirectEnlistmentRevoked",
        "FacWarLPDisqualifiedEvent",
        "FacWarLPDisqualifiedKill",
        "FacWarLPPayoutEvent",
        "FacWarLPPayoutKill",
        "FreelanceProjectClosed",
        "FreelanceProjectCompleted",
        "FreelanceProjectCreated",
        "FreelanceProjectExpired",
        "FreelanceProjectLimitReached",
        "FreelanceProjectParticipantKicked",
        "GameTimeAdded",
        "GameTimeReceived",
        "GameTimeSent",
        "GiftReceived",
        "IHubDestroyedByBillFailure",
        "IncursionCompletedMsg",
        "IndustryOperationFinished",
        "IndustryTeamAuctionLost",
        "IndustryTeamAuctionWon",
        "InfrastructureHubBillAboutToExpire",
        "InsuranceExpirationMsg",
        "InsuranceFirstShipMsg",
        "InsuranceInvalidatedMsg",
        "InsuranceIssuedMsg",
        "InsurancePayoutMsg",
        "InvasionCompletedMsg",
        "InvasionSystemLogin",
        "InvasionSystemStart",
        "JumpCloneDeletedMsg1",
        "JumpCloneDeletedMsg2",
        "KillReportFinalBlow",
        "KillReportVictim",
        "KillRightAvailable",
        "KillRightAvailableOpen",
        "KillRightEarned",
        "KillRightUnavailable",
        "KillRightUnavailableOpen",
        "KillRightUsed",
        "LPAutoRedeemed",
        "LocateCharMsg",
        "MadeWarMutual",
        "MercOfferRetractedMsg",
        "MercOfferedNegotiationMsg",
        "MercenaryDenAttacked",
        "MercenaryDenNewMTO",
        "MercenaryDenReinforced",
        "MissionCanceledTriglavian",
        "MissionOfferExpirationMsg",
        "MissionTimeoutMsg",
        "MoonminingAutomaticFracture",
        "MoonminingExtractionCancelled",
        "MoonminingExtractionFinished",
        "MoonminingExtractionStarted",
        "MoonminingLaserFired",
        "MutualWarExpired",
        "MutualWarInviteAccepted",
        "MutualWarInviteRejected",
        "MutualWarInviteSent",
        "NPCStandingsGained",
        "NPCStandingsLost",
        "OfferToAllyRetracted",
        "OfferedSurrender",
        "OfferedToAlly",
        "OfficeLeaseCanceledInsufficientStandings",
        "OldLscMessages",
        "OperationFinished",
        "OrbitalAttacked",
        "OrbitalReinforced",
        "OwnershipTransferred",
        "RaffleCreated",
        "RaffleExpired",
        "RaffleFinished",
        "ReimbursementMsg",
        "ResearchMissionAvailableMsg",
        "RetractsWar",
        "SPAutoRedeemed",
        "SeasonalChallengeCompleted",
        "SkinSequencingCompleted",
        "SkyhookDeployed",
        "SkyhookDestroyed",
        "SkyhookLostShields",
        "SkyhookOnline",
        "SkyhookUnderAttack",
        "SovAllClaimAquiredMsg",
        "SovAllClaimLostMsg",
        "SovCommandNodeEventStarted",
        "SovCorpBillLateMsg",
        "SovCorpClaimFailMsg",
        "SovDisruptorMsg",
        "SovStationEnteredFreeport",
        "SovStructureDestroyed",
        "SovStructureReinforced",
        "SovStructureSelfDestructCancel",
        "SovStructureSelfDestructFinished",
        "SovStructureSelfDestructRequested",
        "SovereigntyIHDamageMsg",
        "SovereigntySBUDamageMsg",
        "SovereigntyTCUDamageMsg",
        "StationAggressionMsg1",
        "StationAggressionMsg2",
        "StationConquerMsg",
        "StationServiceDisabled",
        "StationServiceEnabled",
        "StationStateChangeMsg",
        "StoryLineMissionAvailableMsg",
        "StructureAnchoring",
        "StructureCourierContractChanged",
        "StructureDestroyed",
        "StructureFuelAlert",
        "StructureImpendingAbandonmentAssetsAtRisk",
        "StructureItemsDelivered",
        "StructureItemsMovedToSafety",
        "StructureLostArmor",
        "StructureLostShields",
        "StructureLowReagentsAlert",
        "StructureNoReagentsAlert",
        "StructureOnline",
        "StructurePaintPurchased",
        "StructureServicesOffline",
        "StructureUnanchoring",
        "StructureUnderAttack",
        "StructureWentHighPower",
        "StructureWentLowPower",
        "StructuresJobsCancelled",
        "StructuresJobsPaused",
        "StructuresReinforcementChanged",
        "TowerAlertMsg",
        "TowerResourceAlertMsg",
        "TransactionReversalMsg",
        "TutorialMsg",
        "WarAdopted ",
        "WarAllyInherited",
        "WarAllyOfferDeclinedMsg",
        "WarConcordInvalidates",
        "WarDeclared",
        "WarEndedHqSecurityDrop",
        "WarHQRemovedFromSpace",
        "WarInherited",
        "WarInvalid",
        "WarRetracted",
        "WarRetractedByConcord",
        "WarSurrenderDeclinedMsg",
        "WarSurrenderOfferMsg",
    ]



"""ESI data providers """

from esi.exceptions import HTTPNotModified
from esi.models import Token
from esi.openapi_clients import ESIClientProvider

from allianceauth.services.hooks import get_extension_logger

from dens import __version__, app_name, compatibility_date, repo_url
from dens.app_settings import DENS_USE_ETAGS
from dens.esi.exceptions import MissingTokenError, NotModified
from dens.esi.models import (
    CharactersCharacterIdAssetsGetItem,
    CharactersCharacterIdAssetsLocationsPostItem,
    CharactersCharacterIdNotificationsGetItem,
)
from dens.models import DenOwner

MERCENARY_DENS_TYPE_IDS = [
    85230,
    85980,
]
TEMPERATE_PLANET_TYPE_ID = 11


esi = ESIClientProvider(
    ua_appname=app_name,
    ua_version=__version__,
    ua_url=repo_url,
    compatibility_date=compatibility_date,
    operations=[
        "GetCharactersCharacterIdAssets",
        "PostCharactersCharacterIdAssetsLocations",
        "GetCharactersCharacterIdNotifications",
    ],
)

logger = get_extension_logger(__name__)


def get_character_assets_from_esi(
    owner: DenOwner,
    *,
    use_etag: bool = True,
) -> list[CharactersCharacterIdAssetsGetItem]:
    """Returns all character assets from the ESI"""
    logger.info(
        "Fetching esi asset for user id '%s' (use etags: '%s')", owner.id, use_etag
    )

    use_etag = use_etag and DENS_USE_ETAGS

    try:
        assets = esi.client.Assets.GetCharactersCharacterIdAssets(
            character_id=owner.character_id,
            token=owner.fetch_token(),
        ).results(
            use_etag=use_etag,
            store_cache=False,
        )
    except Token.DoesNotExist as e:
        logger.error("Missing ESI token for owner id '%s'", owner.id)
        raise MissingTokenError("Missing ESI token when fetching owner assets") from e

    except HTTPNotModified as e:
        raise NotModified("get_character_assets", [owner.character_id]) from e

    logger.debug(assets)

    return [asset.model_dump() for asset in assets]


def get_owner_anchored_dens_from_esi(
    owner: DenOwner,
    *,
    use_etag: bool = True,
) -> list[CharactersCharacterIdAssetsGetItem]:
    """Return all dens locations from the ESI"""

    den_assets = [
        asset
        for asset in get_character_assets_from_esi(owner, use_etag=use_etag)
        if asset["type_id"] in MERCENARY_DENS_TYPE_IDS
        and asset["location_type"] == "solar_system"
    ]

    return den_assets


def get_esi_asset_location(
    owner: DenOwner, item_id: int, *, use_etag: bool = True
) -> tuple[int, int, int]:
    """Returns the position of an item"""
    logger.info(
        "Fetching item id '%s' location from owner '%s' esi information (use etag: '%s')",
        item_id,
        owner,
        use_etag,
    )

    use_etag = not (use_etag and DENS_USE_ETAGS)

    try:
        result: CharactersCharacterIdAssetsLocationsPostItem = (
            esi.client.Assets.PostCharactersCharacterIdAssetsLocations(
                character_id=owner.character_id,
                body=[item_id],
                token=owner.fetch_token(),
            )
            .result(
                use_etag=use_etag,
                store_cache=False,
            )[0]
            .model_dump()
        )
        position = result["position"]
    except Token.DoesNotExist as e:
        logger.error("Missing ESI token for owner id '%s'", owner.id)
        raise MissingTokenError(
            "Missing ESI token when fetching owner asset location"
        ) from e

    logger.debug(position)

    return position["x"], position["y"], position["z"]


def get_owner_notifications(
    owner: DenOwner,
    *,
    use_etag: bool = True,
) -> list[CharactersCharacterIdNotificationsGetItem]:
    """Returns notifications from an owner"""
    logger.info(
        "Fetching notifications for den owner id '%s' with use_etag set to '%s'",
        owner.id,
        use_etag,
    )
    use_etag = use_etag and DENS_USE_ETAGS
    try:
        notifications = esi.client.Character.GetCharactersCharacterIdNotifications(
            character_id=owner.character_id,
            token=owner.fetch_token(),
        ).results(use_etag=use_etag, store_cache=False)
    except Token.DoesNotExist as e:
        logger.error("Missing ESI token for owner '%s'", owner.id)
        raise MissingTokenError(
            "Missing ESI token when fetching owner notifications"
        ) from e
    except HTTPNotModified as e:
        raise NotModified("get_character_notifications", [owner.character_id]) from e

    logger.debug(notifications)

    return [notification.model_dump() for notification in notifications]


def get_owner_mercenarydenreinforced_notifications_from_esi(
    owner: DenOwner,
    *,
    use_etag: bool = True,
) -> list[CharactersCharacterIdNotificationsGetItem]:
    """Returns only the `MercenaryDenReinforced` notifications from this owner"""

    return [
        notification
        for notification in get_owner_notifications(owner, use_etag=use_etag)
        if notification["type"] == "MercenaryDenReinforced"
    ]



"""Views."""

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.db.models import F
from django.http import HttpRequest
from django.shortcuts import get_object_or_404, redirect, render
from django.utils.translation import gettext_lazy as _
from esi.decorators import token_required

from allianceauth.framework.datatables import DataTablesView
from allianceauth.services.hooks import get_extension_logger

from dens import tasks
from dens.models import ESI_SCOPES, DenOwner, MercenaryDen

logger = get_extension_logger(__name__)


@login_required
@permission_required("dens.basic_access")
def index(request):
    """Render index view."""
    return redirect("dens:dens")


@login_required
@permission_required("dens.basic_access")
@token_required(scopes=ESI_SCOPES)
def add_owner(request, token):
    """View to add an owner"""
    character_ownership = get_object_or_404(
        request.user.character_ownerships.select_related("character"),
        character__character_id=token.character_id,
    )

    owner, created = DenOwner.objects.get_or_create(
        character_ownership=character_ownership
    )

    if not created:
        owner.enable()
        messages.success(
            request,
            _(
                "Successfully enabled owner %(owner_name)s. Starting to fetch mercenary dens."
            )
            % {"owner_name": owner.character_name},
        )
    else:
        messages.success(
            request,
            _(
                "Successfully created owner %(owner_name)s. Starting to fetch mercenary dens."
            )
            % {"owner_name": owner.character_name},
        )

    tasks.update_owner_dens.delay(owner.id)

    return redirect("dens:index")


def _get_dens_filtered(request: HttpRequest):
    """Filters mercenary dens based on request/user and applies select_related"""

    user = request.user
    if user.has_perm("dens.manager"):
        qs = MercenaryDen.objects.all()
    elif user.has_perm("dens.alliance_view"):
        alliance = request.user.profile.main_character.alliance
        qs = MercenaryDen.objects.filter_alliance_dens(alliance)
    elif user.has_perm("dens.corporation_view"):
        corporation = request.user.profile.main_character.corporation
        qs = MercenaryDen.objects.filter_corporation_dens(corporation)
    else:
        qs = MercenaryDen.objects.filter_user_dens(user)

    qs = qs.select_related(
        "owner",
        "location",
        "location__solar_system",
        "location__solar_system__constellation",
        "location__solar_system__constellation__region",
    )

    return qs


@permission_required("dens.basic_access")
def dens(request):
    """Renders dens table"""
    qs = _get_dens_filtered(request)

    den_owners = (
        qs.values(
            "owner__character_ownership__character__character_name",
            "owner__character_ownership__character__character_id",
        )
        .annotate(
            character_name=F("owner__character_ownership__character__character_name"),
            character_id=F("owner__character_ownership__character__character_id"),
        )
        .order_by("character_name")
        .distinct()
    )

    den_owner_mains = (
        qs.values(
            "owner__character_ownership__user__profile__main_character__character_name",
            "owner__character_ownership__user__id",
        )
        .annotate(
            name=F(
                "owner__character_ownership__user__profile__main_character__character_name"
            ),
            id=F("owner__character_ownership__user__id"),
        )
        .order_by("name")
        .distinct()
    )

    solar_systems = (
        qs.values(
            "location__solar_system__name",
            "location__solar_system__id",
        )
        .annotate(
            name=F("location__solar_system__name"),
            id=F("location__solar_system__id"),
        )
        .order_by("name")
        .distinct()
    )

    constellations = (
        qs.values(
            "location__solar_system__constellation__name",
            "location__solar_system__constellation__id",
        )
        .annotate(
            name=F("location__solar_system__constellation__name"),
            id=F("location__solar_system__constellation__id"),
        )
        .order_by("name")
        .distinct()
    )

    regions = (
        qs.values(
            "location__solar_system__constellation__region__name",
            "location__solar_system__constellation__region__id",
        )
        .annotate(
            name=F("location__solar_system__constellation__region__name"),
            id=F("location__solar_system__constellation__region__id"),
        )
        .order_by("name")
        .distinct()
    )

    context = {
        "den_owners": den_owners,
        "den_owner_mains": den_owner_mains,
        "solar_systems": solar_systems,
        "constellations": constellations,
        "regions": regions,
    }

    return render(request, "dens/dens.html", context)


class MercenaryDenTableData(
    DataTablesView, LoginRequiredMixin, PermissionRequiredMixin
):
    """Data to populate the mercenary dens datatable"""

    model = MercenaryDen
    permission_required = "dens.basic_access"
    columns = [
        ("owner", "{{ row.owner.character_ownership.character.character_name }}"),
        (
            "owner__character_ownership__user",
            "{{ row.owner.character_ownership.user.profile.main_character.character_name }}",
        ),
        ("location", "{{ row.location.name }}"),
        ("location__solar_system", "dens/stubs/system_name.html"),
        ("", "dens/stubs/region_constellation.html"),
        ("", "dens/stubs/reinforced.html"),  # TODO check for full data
        (
            "location__solar_system__constellation",
            "{{ row.location.solar_system.constellation.name }}",
        ),
        (
            "location__solar_system__constellation__region",
            "{{ row.location.solar_system.constellation.region.name }}",
        ),
    ]

    def get_model_qs(self, request: HttpRequest, *args, **kwargs):
        qs = _get_dens_filtered(request)

        get_params = request.GET.dict()

        if filter_den_owner := get_params.get("filter_den_owner"):
            qs = qs.filter(den_owner_id=filter_den_owner)
        if filter_den_owner_main := get_params.get("filter_den_owner_main"):
            qs = qs.filter(
                den_owner__character_ownership__user_id=filter_den_owner_main
            )
        if filter_solar_system := get_params.get("filter_solar_system"):
            qs = qs.filter(location__solar_system_id=filter_solar_system)
        if filter_constellation := get_params.get("filter_constellation"):
            qs = qs.filter(
                location__solar_system__constellation_id=filter_constellation
            )
        if filter_region := get_params.get("filter_region"):
            qs = qs.filter(location__solar_system__constellation__region=filter_region)
        if filter_reinforced := get_params.get("filter_reinforced"):
            if filter_reinforced == "YES":
                qs = qs.filter_is_reinforced(True)
            elif filter_reinforced == "NO":
                qs = qs.filter_is_reinforced(False)
            logger.warning(
                "Unexpected reinforcement filter received: '%s'", filter_reinforced
            )

        return qs



"""Esi related exceptions"""

from dataclasses import dataclass
from typing import Any


class MissingTokenError(Exception):
    """Raised when the token for this user isn't found"""


@dataclass
class NotModified(Exception):
    """Raised when a non modified exception was thrown by the provider. Resource didn't change"""

    operation_name: str
    args: list | None = None
    body: Any | None = None



# Generated by Django 5.2.12 on 2026-03-20 22:06

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("dens", "0004_discordwebhook"),
        ("eve_sde", "0018_blueprintactivity_blueprintactivityproduct_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mercenaryden",
            name="location",
            field=models.OneToOneField(
                help_text="Location of the den",
                on_delete=django.db.models.deletion.CASCADE,
                to="eve_sde.planet",
            ),
        ),
    ]



# Generated by Django 4.2.21 on 2025-08-04 07:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("dens", "0003_alter_mercenaryden_location"),
    ]

    operations = [
        migrations.CreateModel(
            name="DiscordWebhook",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "webhook_url",
                    models.CharField(
                        help_text="Webhook URL", max_length=255, unique=True
                    ),
                ),
                ("is_enabled", models.BooleanField(default=True)),
                (
                    "name",
                    models.CharField(
                        help_text="Text to recognize the webhook",
                        max_length=30,
                        unique=True,
                    ),
                ),
            ],
        ),
    ]



from django.contrib.auth.models import Permission, User

from allianceauth.notifications import notify

from .django import users_with_permission


def notify_admins(message: str, title: str, level: str = "info") -> None:
    """Send notification to all admins.

    Args:
        message: Message text
        title: Message title
        level: Notification level of the message.
    """
    try:
        perm = Permission.objects.get(codename="logging_notifications")
    except Permission.DoesNotExist:
        users = User.objects.filter(is_superuser=True)
    else:
        users = users_with_permission(perm)
    for user in users:
        notify(user, title=title, message=message, level=level)



"""Mercenary dens plugin app for Alliance Auth."""

# pylint: disable = invalid-name

from datetime import date

from .app_settings import DENS_CUSTOM_COMPATIBILITY_DATE

__version__ = "2.0.0"
app_name = "aa-dens"
repo_url = "https://gitlab.com/r0kym/aa-dens"
if DENS_CUSTOM_COMPATIBILITY_DATE:
    compatibility_date = DENS_CUSTOM_COMPATIBILITY_DATE
else:
    compatibility_date = date(2026, 3, 20)



from django.apps import apps
from django.contrib.auth.models import Permission, User
from django.db import models


def app_labels() -> set:
    """returns set of all current app labels"""
    return set(apps.app_configs.keys())


def users_with_permission(
    permission: Permission, include_superusers=True
) -> models.QuerySet:
    """returns queryset of users that have the given Django permission

    Args:
        permission: required permission
        include_superusers: whether superusers are included in the returned list
    """
    users_qs = (
        permission.user_set.all()
        | User.objects.filter(
            groups__in=list(permission.group_set.values_list("pk", flat=True))
        )
        | User.objects.select_related("profile").filter(
            profile__state__in=list(permission.state_set.values_list("pk", flat=True))
        )
    )
    if include_superusers:
        users_qs |= User.objects.filter(is_superuser=True)
    return users_qs.distinct()



from datetime import timedelta

from django.utils import timezone

from allianceauth.eveonline.models import (
    EveAllianceInfo,
    EveCharacter,
    EveCorporationInfo,
)

from dens.models import DenOwner, MercenaryDen, MercenaryDenReinforcedNotification
from dens.tests.utils import (
    DensTestCase,
    create_fake_den_owners,
    create_fake_dens_for_owner,
)


class TestNotification(DensTestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        owners = create_fake_den_owners(2)

        create_fake_dens_for_owner(owners[0], [40255090, 40255092])
        create_fake_dens_for_owner(owners[1], [40255094])

    def test_pre_conditions(self):
        """Test case to check that the situations is as expected"""
        self.assertEqual(DenOwner.objects.count(), 2)
        self.assertEqual(MercenaryDen.objects.count(), 3)

        self.assertTrue(EveCorporationInfo.objects.filter(corporation_id=2000).exists())
        self.assertTrue(EveCorporationInfo.objects.filter(corporation_id=2001).exists())

        self.assertTrue(EveAllianceInfo.objects.filter(alliance_id=3000).exists())
        self.assertTrue(EveAllianceInfo.objects.filter(alliance_id=3001).exists())

    def test_get_all_dens(self):
        self.assertEqual(MercenaryDen.objects.all().count(), 3)

    def test_get_alliance_dens(self):
        alliance_1 = EveAllianceInfo.objects.get(alliance_id=3000)
        alliance_2 = EveAllianceInfo.objects.get(alliance_id=3001)

        self.assertEqual(
            MercenaryDen.objects.filter_alliance_dens(alliance_1).count(), 2
        )
        self.assertEqual(
            MercenaryDen.objects.filter_alliance_dens(alliance_2).count(), 1
        )

    def test_get_corporation_dens(self):
        corporation_1 = EveCorporationInfo.objects.get(corporation_id=2000)
        corporation_2 = EveCorporationInfo.objects.get(corporation_id=2001)

        self.assertEqual(
            MercenaryDen.objects.filter_corporation_dens(corporation_1).count(), 2
        )
        self.assertEqual(
            MercenaryDen.objects.filter_corporation_dens(corporation_2).count(), 1
        )

    def test_reinforce_filter(self):
        char = EveCharacter.objects.all()[0]
        now = timezone.now()
        dens = MercenaryDen.objects.all()

        # future den timer
        MercenaryDenReinforcedNotification.objects.create(
            id=2,
            den=dens[0],
            reinforced_by=char,
            enter_reinforcement=now - timedelta(hours=1),
            exit_reinforcement=now + timedelta(days=1),
        )
        # passed den timer
        MercenaryDenReinforcedNotification.objects.create(
            id=1,
            den=dens[1],
            reinforced_by=char,
            enter_reinforcement=now - timedelta(days=2),
            exit_reinforcement=now - timedelta(days=1),
        )
        # one den has no timers

        reinforce_true = MercenaryDen.objects.filter_is_reinforced(True)
        reinforce_false = MercenaryDen.objects.filter_is_reinforced(False)

        self.assertEqual(reinforce_true.count(), 1)
        self.assertEqual(reinforce_false.count(), 2)

        self.assertIn(dens[0], reinforce_true)
        self.assertIn(dens[1], reinforce_false)
        self.assertIn(dens[2], reinforce_false)



"""Models."""

import datetime

import dhooks_lite
from eve_sde.models import Planet

from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from esi.models import Token

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.evelinks.eveimageserver import type_icon_url
from allianceauth.eveonline.models import EveCharacter
from allianceauth.services.hooks import get_extension_logger

from dens.managers import (
    DenOwnerManager,
    DiscordWebhookManager,
    MercenaryDenManager,
    MercenaryDenUnderAttackNotificationManager,
    MercernaryDenReinforcedNotificationManager,
)

ESI_SCOPES = [
    "esi-assets.read_assets.v1",
    "esi-characters.read_notifications.v1",
]

logger = get_extension_logger(__name__)


class General(models.Model):
    """A meta model for app permissions."""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_access", "Can access this app"),
            (
                "corporation_view",
                "Can view all dens anchored by members of their corporation",
            ),
            (
                "alliance_view",
                "Can view all dens anchored by members of their alliance",
            ),
            ("manager", "Can see all user's mercenary dens"),
        )


class DenOwner(models.Model):
    """Represents a character that will drop mercenary dens"""

    objects = DenOwnerManager()

    character_ownership = models.ForeignKey(
        CharacterOwnership,
        on_delete=models.CASCADE,
        related_name="+",
        help_text="Character used to sync mercenary dens",
    )

    is_enabled = models.BooleanField(
        default=True, db_index=True, help_text="Disabled characters won't be synced"
    )

    def __str__(self):
        return self.character_name

    def fetch_token(self) -> Token:
        """Return valid token for this mining corp or raise exception on any error."""
        if not self.character_ownership:
            raise RuntimeError("This owner has no character configured.")
        token = (
            Token.objects.filter(
                character_id=self.character_ownership.character.character_id
            )
            .require_scopes(ESI_SCOPES)
            .require_valid()
            .first()
        )
        if not token:
            raise Token.DoesNotExist(f"{self}: No valid token found.")
        return token

    @property
    def character_name(self) -> str:
        """Returns the character name"""
        return self.character_ownership.character.character_name

    @property
    def user_name(self) -> str:
        """Returns the user name"""
        return self.character_ownership.user.username

    @property
    def character_id(self) -> int:
        """Returns the character id"""
        return self.character_ownership.character.character_id

    def enable(self):
        """Sets an owner back to activity"""
        self.is_enabled = True
        self.save()

    def disable(self):
        """Disables an owner"""
        self.is_enabled = False
        self.save()


class MercenaryDen(models.Model):
    """Represents anchored mercenary dens"""

    objects = MercenaryDenManager()

    id = models.BigIntegerField(
        primary_key=True, help_text=_("Eve online id of the den")
    )

    owner = models.ForeignKey(
        DenOwner,
        on_delete=models.CASCADE,
        help_text=_("Character that anchored the den"),
    )
    location = models.OneToOneField(
        Planet, on_delete=models.CASCADE, help_text=_("Location of the den")
    )

    def __str__(self) -> str:
        return f"Den {self.location.name}"

    @property
    def is_reinforced(self) -> bool:
        """True if there's an unexited reinforcement notification"""
        now = timezone.now()
        return MercenaryDenReinforcedNotification.objects.filter(
            den=self, exit_reinforcement__gt=now
        ).exists()

    @property
    def reinforcement_time(self) -> datetime.datetime | None:
        """Return the den reinforcement time if it exists"""
        now = timezone.now()
        try:
            notification = MercenaryDenReinforcedNotification.objects.get(
                den=self, exit_reinforcement__gt=now
            )
            return notification.exit_reinforcement
        except MercenaryDenReinforcedNotification.DoesNotExist:
            pass
        return None


class MercenaryDenReinforcedNotification(models.Model):
    """Represents the notification of an owner den reinforced"""

    objects = MercernaryDenReinforcedNotificationManager()

    id = models.BigIntegerField(primary_key=True)

    den = models.ForeignKey(MercenaryDen, on_delete=models.SET_NULL, null=True)

    reinforced_by = models.ForeignKey(
        EveCharacter,
        on_delete=models.CASCADE,
        help_text="Character that reinforced the Mercenary Den",
    )
    enter_reinforcement = models.DateTimeField(
        help_text=_("Timer when the den was reinforced")
    )
    exit_reinforcement = models.DateTimeField(
        help_text=_("Timer when the den will leave reinforcement")
    )

    def __str__(self) -> str:
        return f"Den {self.den.location.name} reinforced by {self.reinforced_by.character_name}"

    def is_in_future(self) -> bool:
        """True if the timer is in the future"""
        return self.exit_reinforcement > timezone.now()


class MercenaryDenUnderAttackNotification(models.Model):
    """Represents the notification of an owner den under attack"""

    objects = MercenaryDenUnderAttackNotificationManager()

    id = models.BigIntegerField(primary_key=True)

    den = models.ForeignKey(MercenaryDen, on_delete=models.SET_NULL, null=True)

    attacked_by = models.ForeignKey(
        EveCharacter,
        on_delete=models.SET_NULL,
        null=True,
        help_text="Character that attacked the Mercenary Den",
    )
    attack_time = models.DateTimeField(
        null=True,
        help_text=_("Timestamp when the den was attacked"),
    )
    raw_text = models.TextField(blank=True)

    def __str__(self) -> str:
        if self.den:
            return f"Den {self.den.location.name} under attack"
        return f"Den under attack (notification {self.id})"


class DiscordWebhook(models.Model):
    """Discord webhook to route notifications to"""

    objects = DiscordWebhookManager()

    webhook_url = models.CharField(
        max_length=255, unique=True, help_text=_("Webhook URL")
    )

    is_enabled = models.BooleanField(default=True)
    name = models.CharField(
        max_length=30, unique=True, help_text="Text to recognize the webhook"
    )

    # pylint: disable = too-many-positional-arguments, too-many-arguments
    def send_info(
        self,
        title: str,
        content: str,
        fields=None,
        author_name: str = None,
        author_icon_url: str = None,
    ):
        """Sends an information through the webhook"""
        embed = self._build_embed(
            title, content, 0x5CDBF0, fields, author_name, author_icon_url
        )
        self._send_through_webhook([embed])

    # pylint: disable = too-many-positional-arguments, too-many-arguments
    def send_alert(
        self,
        title: str,
        content: str,
        fields=None,
        author_name: str = None,
        author_icon_url: str = None,
    ):
        """Sends an alert through the webhook"""
        embed = self._build_embed(
            title, content, 0xFF5733, fields, author_name, author_icon_url
        )
        self._send_through_webhook([embed])

    def _build_webhook(self) -> dhooks_lite.Webhook:
        """Builds the discord webhook and returns it"""
        return dhooks_lite.Webhook(
            self.webhook_url,
            username="Den notification",
            avatar_url="https://gitlab.com/uploads/-/system/project/avatar/64868536/tents_1_.png",
        )

    def _send_through_webhook(self, embeds: list[dhooks_lite.Embed]):
        """Sends embeds through the webhook"""
        webhook = self._build_webhook()
        webhook.execute(embeds=embeds)

    # pylint: disable = too-many-positional-arguments, too-many-arguments
    def _build_embed(
        self,
        title: str,
        content: str,
        color: int,
        fields=None,
        author_name: str = None,
        author_icon_url: str = None,
    ) -> dhooks_lite.Embed:
        """Build a discord embedd and returns it"""

        formatted_fields = []
        if fields:
            for field in fields:
                formatted_fields.append(dhooks_lite.Field(field[0], str(field[1])))
        if not author_name:
            author_name = "AA-Dens"

        embed = dhooks_lite.Embed(
            author=dhooks_lite.Author(
                name=author_name,
                icon_url=author_icon_url,
            ),
            title=title,
            description=content,
            thumbnail=dhooks_lite.Thumbnail(type_icon_url(85230, size=128)),
            color=color,
            fields=formatted_fields,
            timestamp=datetime.datetime.now(datetime.timezone.utc),
            footer=dhooks_lite.Footer(
                "aa-dens",
            ),
        )

        return embed



"""App settings."""

from datetime import date

from django.conf import settings

from dens.vendor.app_utils.app_settings import clean_setting

DENS_ADMIN_NOTIFICATIONS_ENABLED = clean_setting(
    "DENS_ADMIN_NOTIFICATIONS_ENABLED", True
)

DENS_FORWARD_TIMER_TO_TIMERBOARD = clean_setting(
    "DENS_FORWARD_TIMER_TO_TIMERBOARD", True
)

DENS_USE_ETAGS = clean_setting(
    "DENS_USE_ETAGS",
    True,
)


DENS_CUSTOM_COMPATIBILITY_DATE: date | str | None = getattr(
    settings, "DENS_CUSTOM_COMPATIBILITY_DATE", None
)



# Generated by Django 4.2.29 on 2026-04-12 17:43

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveonline", "0017_alliance_and_corp_names_are_not_unique"),
        ("dens", "0001_initialize"),
    ]

    operations = [
        migrations.CreateModel(
            name="MercenaryDenUnderAttackNotification",
            fields=[
                ("id", models.BigIntegerField(primary_key=True, serialize=False)),
                (
                    "attack_time",
                    models.DateTimeField(
                        help_text="Timestamp when the den was attacked", null=True
                    ),
                ),
                ("raw_text", models.TextField(blank=True)),
                (
                    "attacked_by",
                    models.ForeignKey(
                        help_text="Character that attacked the Mercenary Den",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="eveonline.evecharacter",
                    ),
                ),
                (
                    "den",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="dens.mercenaryden",
                    ),
                ),
            ],
        ),
    ]
